# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import division
from future import standard_library
from builtins import *
from netaddr.ip import *
from qwhois.whois import NICClient

import re
import os
import subprocess
import socket

standard_library.install_aliases()
suffixes = None


def _is_ip(url):
    """
    判断是否为ip.
    """
    try:
        IPAddress(url)
        return True
    except:
        return False


# 判断是否全是ASCII码
def _is_ascii(value):
    """
    判断是否为ascii码
    """
    for c in value:
        if ord(c) > 0x7f:
            return False
    return True


def whois(url, command=False, flags=0, timeout=10):
    """
    获取whois信息.
    url: 域名/IP/链接
    command: 是否调用命令行
    flags: 查询whois的资源
    timeout: 超时查询
    """
    # 判断域名是否为ip.
    if _is_ip(url):
        result = socket.gethostbyaddr(url)
        url = result[0]
    domain = extract_domain(url)

    if command:
        # try native whois command
        r = subprocess.Popen(['whois', domain], stdout=subprocess.PIPE)
        text = r.stdout.read().decode()
    else:
        # try builtin client
        nic_client = NICClient()
        text = nic_client.whois_lookup(domain, flags, timeout)

    return text


def extract_domain(url):
    """Extract the domain from the given URL

    >>> print(extract_domain('http://www.google.com.au/tos.html'))
    google.com.au
    >>> print(extract_domain('abc.def.com'))
    def.com
    >>> print(extract_domain(u'www.公司.hk'))
    公司.hk
    >>> print(extract_domain('chambagri.fr'))
    chambagri.fr
    >>> print(extract_domain('www.webscraping.com'))
    webscraping.com
    >>> print(extract_domain('198.252.206.140'))
    stackoverflow.com
    >>> print(extract_domain('102.112.2O7.net'))
    2o7.net
    >>> print(extract_domain('globoesporte.globo.com'))
    globo.com
    >>> print(extract_domain('1-0-1-1-1-0-1-1-1-1-1-1-1-.0-0-0-0-0-0-0-0-0-0-0-0-0-10-0-0-0-0-0-0-0-0-0-0-0-0-0.info'))
    0-0-0-0-0-0-0-0-0-0-0-0-0-10-0-0-0-0-0-0-0-0-0-0-0-0-0.info
    >>> print(extract_domain('2607:f8b0:4006:802::200e'))
    1e100.net
    >>> print(extract_domain('172.217.3.110'))
    1e100.net
    """

    # load known TLD suffixes
    global suffixes
    if not suffixes:
        # downloaded from https://publicsuffix.org/list/public_suffix_list.dat
        tlds_path = os.path.join(os.getcwd(), os.path.dirname(__file__), 'data', 'public_suffix_list.dat')
        with open(tlds_path, encoding='utf-8') as tlds_fp:
            suffixes = set(
                line.encode('utf-8') for line in tlds_fp.read().splitlines() if line and not line.startswith('//')
            )

    if not isinstance(url, str):
        url = url.decode('utf-8')
    url = re.sub('^.*://', '', url)
    url = url.split('/')[0].lower()

    # find the longest suffix match
    url = to_utf8(url)
    domain = b''
    for section in reversed(url.split('.')):
        if domain:
            domain = b'.' + domain
        domain = section.encode('utf-8') + domain

        if domain not in suffixes:
            break
    # print("extract_domain: ", domain.decode('utf-8'))

    return to_punycode(domain.decode('utf-8'))


# 域名编码
def to_punycode(url):
    """
    将url或域名转换为punycode码
    url: 域名或者链接.
    """
    domain = b''
    for section in reversed(url.split('.')):
        if domain:
            domain = b'.' + domain

        if not _is_ascii(section):
            section = b'xn--' + section.encode('punycode')
        else:
            section = section.encode('utf-8')
        domain = section + domain

    return domain.decode('utf-8')


def to_utf8(url):
    """
    将punycode码转为utf8格式的字符串
    url 连接地址
    """
    domain = b''
    for section in reversed(url.split('.')):
        if domain:
            domain = b'.' + domain

        if section.find('xn--') == 0:
            section = section.replace('xn--', '').encode('utf-8').decode('punycode').encode('utf-8')
        else:
            section = section.encode('utf-8')
        domain = section + domain

    return domain.decode('utf-8')



# -*- coding: utf-8 -*-

"""
Whois client for python

transliteration of:
http://www.opensource.apple.com/source/adv_cmds/adv_cmds-138.1/whois/whois.c

Copyright (c) 2010 Chris Wolf

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

import json
import os
import re
import socket


class NICClient(object):

    # 特殊服务器地址
    ANICHOST = "whois.arin.net"
    LNICHOST = "whois.lacnic.net"
    RNICHOST = "whois.ripe.net"
    PNICHOST = "whois.apnic.net"
    QNICHOST_TAIL = ".whois-servers.net"
    BNICHOST = "whois.registro.br"
    NORIDHOST = "whois.norid.no"
    PANDIHOST = "whois.pandi.or.id"
    DENICHOST = "de.whois-servers.net"
    HR_HOST = "whois.dns.hr"
    DK_HOST = "whois.dk-hostmaster.dk"
    WHOIS_SERVER_MAP = None
    # whois服务默认端口43
    WHOIS_SERVER_PORT = 43

    # CA_HOST = "whois.ca.fury.ca"
    # PE_HOST = "kero.yachay.pe"
    WHOIS_RECURSE = 0x01
    WHOIS_QUICK = 0x02

    ip_whois = [LNICHOST, RNICHOST, PNICHOST, BNICHOST, PANDIHOST]

    @staticmethod
    def find_whois_server(buf, hostname, query):
        """Search the initial TLD lookup results for the regional-specifc
        whois server for getting contact details.
        """
        host = None
        match = re.compile('Domain Name: {}\s*.*?Whois Server: (.*?)\s'.format(query), flags=re.IGNORECASE | re.DOTALL).search(buf)
        if match:
            host = match.groups()[0]
            # if the whois address is domain.tld/something then
            # s.connect((hostname, 43)) does not work
            if host.count('/') > 0:
                host = None
        elif hostname == NICClient.ANICHOST:
            for nic_host in NICClient.ip_whois:
                if buf.find(nic_host) != -1:
                    host = nic_host
                    break
        return host

    @staticmethod
    def choose_server(domain):
        # load whois server map
        if not NICClient.WHOIS_SERVER_MAP:
            server_path = os.path.join(os.getcwd(), os.path.dirname(__file__), 'data', 'whois_server.map')
            with open(server_path, encoding='utf-8') as tlds_fp:
                NICClient.WHOIS_SERVER_MAP = json.loads(tlds_fp.read())

        if domain.endswith("-NORID"):
            return NICClient.NORIDHOST
        if domain.endswith("id"):
            return NICClient.PANDIHOST
        if domain.endswith("hr"):
            return NICClient.HR_HOST

        domain = domain.split('.')
        if len(domain) < 2:
            return None
        tld = domain[-1]
        if tld[0].isdigit():
            return NICClient.ANICHOST
        elif tld in NICClient.WHOIS_SERVER_MAP:
            return NICClient.split_host(NICClient.WHOIS_SERVER_MAP[tld])
        else:
            return tld + NICClient.QNICHOST_TAIL

    @staticmethod
    def split_host(host):
        hosts = host.split(':')
        if len(hosts) >= 2:
            NICClient.WHOIS_SERVER_PORT = hosts[-1]
            return hosts[-2]
        else:
            return host

    def whois(self, domain, hostname, flags, many_results=False, port=43, timeout=10):
        response = b''
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(timeout)
        try:
            s.connect((hostname, port))
            if hostname == NICClient.DENICHOST:
                domain_bytes = "-T dn,ace -C UTF-8 " + domain
            elif hostname == NICClient.DK_HOST:
                domain_bytes = " --show-handles " + domain
            elif hostname.endswith(NICClient.QNICHOST_TAIL) and many_results:
                domain_bytes = '=' + domain
            else:
                domain_bytes = domain
            s.send(bytes(domain_bytes, 'utf-8') + b"\r\n")

            # recv returns bytes
            while True:
                d = s.recv(4096)
                response += d
                if not d:
                    break
            s.close()

            host = None
            response = response.decode('utf-8', 'replace')
            if 'with "=xxx"' in response:
                return self.whois(domain, hostname, flags, True, port=port, timeout=timeout)
            if flags & NICClient.WHOIS_RECURSE and host is None:
                host = NICClient.find_whois_server(response, hostname, domain)
            if host is not None:
                response += self.whois(domain, host, 0, port=port, timeout=timeout)
        except socket.error as exc:
            response = "Error: trying to connect to socket: " + repr(exc)
            s.close()
        return response

    def whois_lookup(self, domain, flags, timeout):
        NICClient.WHOIS_SERVER_PORT = 43
        host = NICClient.choose_server(domain)
        if host is not None:
            result = self.whois(domain, host, flags, port=NICClient.WHOIS_SERVER_PORT, timeout=timeout)
        else:
            result = 'Error: Not find whois server'

        return result



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

# 设置包的属性. 其他的就是一些引用的问题了.
# whois被别人使用了. 没有办法了. 改个名字.
setuptools.setup(
    # 扩展包名
    name="qwhois",
    # 版本.
    version="1.0.1",
    # 作者
    author="adolph",
    # 作者邮箱
    auth_email="yuan13036395508@gmail.com",
    # 简单描述
    descrition="whois lookup >>> 查询域名的whois信息",
    # 长描述
    long_description=long_description,
    # 描述文件格式
    long_description_content_type="text/markdown",
    # 指定对应的git仓库地址
    url="https://github.com/qgsafe/qwhois",
    # 指定包
    packages=setuptools.find_packages(),
    # 指定语言. 指定开源协议
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # 指定python版本.
    python_requires='>=3.6',
    # 安装时. 安装依赖包.
    install_requires=['future', 'netaddr'],
    # 引入非python文件.
    include_package_data=True
)